<?php
namespace Teknisa\EST\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Util\Functions;
use \Estoque\Entities\Posestdia;
use \Geral\Entities\Paramfilial;
use \Geral\Entities\Prodfili;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Estoque\Entities\Lanctoestoq;
use Teknisa\Model\Estoque\Entities\Ctrlmoviestq;
use Teknisa\Model\Estoque\Entities\Itlanctoest;
use Teknisa\EST\Services\ValidacoesEstoqueService;
use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;
use Teknisa\Libs\Util\XLS\Table;
use Teknisa\Libs\Util\XLS\Exception;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;

class Est57000TransferenciaEstoqueService{

    protected $environment;
    protected $dataSourceManager;
    protected $newCode;
    protected $est57000transferenciaestoquefactory;
    protected $nrorg;

    public function __construct(Environment $environment, \Doctrine\ORM\EntityManager $entityManager ,$dataSourceManager,NewCode $newCode,$est57000transferenciaestoquefactory, $validacoesEstoqueService){
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode = $newCode;
        $this->nrorg = $environment->getNrOrg();
        $this->est57000transferenciaestoquefactory = $est57000transferenciaestoquefactory;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
    }

    public function buscar($condicao){
        return $this->dataSourceManager->findBy($condicao);
    }

    public function deletar($dataSet){
        $this->dataSourceManager->delete($dataSet);
    }

    public function geraNovoCodigo($contador, $nrorg, $interval,$tamanho){
        return $this->newCode->getCodeByTableName($contador, $nrorg ,$interval,$tamanho);
    }

    public function realizaConsultaBanco($conditions,$dataSourceName) {
        $filterCriteria = new FilterCriteria($dataSourceName,$conditions);
        $dataSet = $this->buscar($filterCriteria);
        return $dataSet;
    }

    public function buscaDataProcessamento($row){
        try {
            $nrorg = $this->environment->getNrOrgTrab();
            $data = $this->est57000transferenciaestoquefactory->findDataProcessamento($nrorg, $row['CDFILIAL']);
            return $data[0];
        }
        catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaProdutosRequisicao($transferencia){
        try {
            $produtos = $this->est57000transferenciaestoquefactory->buscaProdutosRequisicao($this->environment->getNrorgTrab(), $transferencia['CDFILIAL'], $transferencia['NRREQUESTO']);
            return $produtos;
        }
        catch (\Exception $e) {
            throw $e;
        }
    }

    public function deleteTransferencia($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            //CASO O TRANSFERENCIA FOI FEITA A PARTIR DE UMA REQUISIÇÃO, ATUALIZA O STATUS DA REQUISICAO
            if($row['NRREQUESTO']){
                //VERIFICA A REQUISIÇÃO COMPLEMENTAR COM LANÇAMENTO DE ESTOQUE
                $result = $this->est57000transferenciaestoquefactory->verificaReqComp($row['CDFILIAL'], $row['CDFILIAL'], $row['NRLANESTSAID'], NULL);
                if(!$result) {
                    //DELETA REQUISIÇÃO COMPLEMENTAR SEM LANÇAMENTO DE ESTOQUE
                    $this->est57000transferenciaestoquefactory->deletarRequisicao($row['CDFILIAL'], $row['CDFILIAL'], $row['NRLANESTSAID']);
                }
                else {
                    //ATUALIZA STATUS DA REQUISIÇÃO COMPLEMENTAR E ATUALIZA ITENS DA REQUISIÇÃO COMPLENTAR COM LANÇAMENTO DE ESTOQUE 
                    $this->updateRequisicaoStatus($row['CDFILIAL'], $result['NRREQUESTO'], 'A');
                    $this->est57000transferenciaestoquefactory->updateItemRequisicao($row['CDFILIAL'], $result['NRREQUESTO']);
                }
                //ATUALIZA STATUS DA REQUISIÇÃO E DOS ITENS DA REQUISIÇÃO
                $this->updateRequisicaoStatus($row['CDFILIAL'], $row['NRREQUESTO'], 'A');
                $params = array(':IDSTITREQU'=> 'A',
                    ':CDFILILANC'=>NULL,
                    ':NRLANCESTQ'=>NULL,
                    ':NRSEQUITEM'=>NULL,
                    ':CDFILIAL'=>$row['CDFILIAL'],
                    ':NRREQUESTO'=>$row['NRREQUESTO'],
                    ':OLD_NRLANCESTQ'=>$row['NRLANESTSAID'],
                    ':CDPRODUTO'=>NULL,
                    ':NRSEQITEM'=>NULL,
                    ':NRORG'=>$row['NRORG']
                );
                $this->est57000transferenciaestoquefactory->updateItrequestoStatus($params);
            }
            $lancsaida = $this->validacoesEstoqueService->buscaLanctoestoq($row['CDFILIAL'], $row['NRLANESTSAID'], $row['NRORG']);
            $this->validacoesEstoqueService->setLanctoestoq(\Estoque\Entities\Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($row['CDFILIAL'], $row['NRLANESTENTR'], $row['NRORG']));
            $this->est57000transferenciaestoquefactory->deleteItlanctoest($this->environment->getNrorgTrab(), $row['CDFILIAL'], $row['NRLANESTENTR'],$row['NRLANESTSAID']);
            $this->est57000transferenciaestoquefactory->deleteCtrlmoviestq($this->environment->getNrorgTrab(), $row['CDFILIAL'], $row['NRCTRLMOVEST']);
            $this->est57000transferenciaestoquefactory->deleteLanctoestoq($this->environment->getNrorgTrab(), $row['CDFILIAL'], $row['NRLANESTENTR'],$row['NRLANESTSAID']);
            $this->validacoesEstoqueService->atualizaEstoque($row['CDFILIAL'], $row['NRLANESTENTR']);
            $this->validacoesEstoqueService->setLanctoestoq($lancsaida);
            $this->validacoesEstoqueService->atualizaEstoque($row['CDFILIAL'], $row['NRLANESTSAID']);
            $connection->commit();

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function deleteProduto($rows){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $produto = $rows['produtos'];
            $transferencia = $rows['transferencia'];
            $itensTransferencia = $this->est57000transferenciaestoquefactory->buscaItensTransferencia($this->environment->getNrorgTrab(), $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID']);

            // NAO SE PODE EXCLUIR O PRODUTO SE ELE FOR O UNICO DA REQUISICAO
            if(count($itensTransferencia) > 1 ){
                $produtosaida = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Itlanctoest')->findOneBy(
                    array(  'nrorg' => $produto['NRORG'],
                            'cdfilial' => $produto['CDFILIAL'],
                            'nrsequitem' => $produto['NRSEQUITEM'],
                            'nrlancestq' => $transferencia['NRLANESTSAID'],
                    )
                );
                $produtoentrada = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Itlanctoest')->findOneBy(
                    array(  'nrorg' => $produto['NRORG'],
                            'cdfilial' => $produto['CDFILIAL'],
                            'nrsequitem' => $produto['NRSEQUITEM'],
                            'nrlancestq' => $transferencia['NRLANESTENTR'],

                    )
                );
              
                
                //CASO O TRANSFERENCIA FOI FEITA A PARTIR DE UMA REQUISIÇÃO, ATUALIZA O STATUS DA REQUISICAO
                if($transferencia['NRREQUESTO']){
                    $prodDaRequisicao = false;
                    //VERIFICA SE O ITEM DA REQUISIÇÃO COMPLEMENTAR TEM LANÇAMENTO DE ESTOQUE
                    $produtosRequisicao = $this->buscaProdutosRequisicao($transferencia);
                    foreach ($produtosRequisicao as $produtoRequisicao) {
                        if($produto['CDPRODUTO'] == $produtoRequisicao['CDPRODUTO']) {
                            $prodDaRequisicao = true;
                        }
                    }
                    if($prodDaRequisicao == true) {
                        $result = $this->est57000transferenciaestoquefactory->verificaReqComp($transferencia['CDFILIAL'], $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $produto['NRSEQITREQC']);
                        if(!$result) {
                            //DELETA O ITEM DA REQUISICAO COMPLEMENTAR SEM LANÇAMENTO DE ESTOQUE
                            $ultimoItem = !$this->est57000transferenciaestoquefactory->verificaUltimoItemReqComp($transferencia['CDFILIAL'], $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $produto['NRSEQITREQC']);
                            if(!$ultimoItem) {
                                $this->est57000transferenciaestoquefactory->deletarRequisicao($transferencia['CDFILIAL'], $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID']);
                            }
                            else {
                                $this->est57000transferenciaestoquefactory->deleteItemReqComp($transferencia['CDFILIAL'], $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $produto['NRSEQUITEM']);
                            }
                        }
                        else {
                            //ATUALIZA STATUS DO ITEM DA REQUISIÇÃO COMPLEMENTAR E ATUALIZA ITENS DA REQUISIÇÃO COMPLENTAR COM LANÇAMENTO DE ESTOQUE 
                            $this->est57000transferenciaestoquefactory->updateItemReqComp($transferencia['CDFILIAL'], $transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $produto['NRSEQITREQC']);
                        }
                        //ATUALIZA STATUS DO ITEM DA REQUISIÇÃO
                        $this->updateRequisicaoStatus($transferencia['CDFILIAL'], $transferencia['NRREQUESTO'], 'A');
                        $params = array(':IDSTITREQU'=> 'A',
                            ':CDFILILANC'=>NULL,
                            ':NRLANCESTQ'=>NULL,
                            ':NRSEQUITEM'=>NULL,
                            ':CDFILIAL'=>$transferencia['CDFILIAL'],
                            ':NRREQUESTO'=>$transferencia['NRREQUESTO'],
                            ':OLD_NRLANCESTQ'=>$transferencia['NRLANESTSAID'],
                            ':CDPRODUTO'=>$produto['CDPRODUTO'], 
                            ':NRSEQITEM' =>NULL,
                            ':NRORG'=>$transferencia['NRORG']
                        );
                        $this->est57000transferenciaestoquefactory->updateItrequestoStatus($params);
                    }
                }

            // Atualiza o estoque
            $lancsaida = $this->validacoesEstoqueService->buscaLanctoestoq($transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $transferencia['NRORG']);
            $this->validacoesEstoqueService->setLanctoestoq(\Estoque\Entities\Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($transferencia['CDFILIAL'], $transferencia['NRLANESTENTR'], $transferencia['NRORG']));
            $this->entityManager->remove($produtosaida);
            $this->entityManager->flush();
            $this->entityManager->remove($produtoentrada);
            $this->entityManager->flush();
            $this->validacoesEstoqueService->atualizaEstoque($transferencia['CDFILIAL'], $transferencia['NRLANESTENTR']);
            $this->validacoesEstoqueService->setLanctoestoq($lancsaida);
            $this->validacoesEstoqueService->atualizaEstoque($transferencia['CDFILIAL'], $transferencia['NRLANESTSAID']);
            $connection->commit();
            }else{
                throw new \Exception("Não foi possível exlcuir. Existe somente este produto cadastrado para a transferência.", 1);
            }
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function editProdutoTransferencia($rows){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $produtos = $rows['produtos'];
            $transferencia = $rows['transferencia'];
            $nrorg       = $this->environment->getNrorgTrab();
            $operador    = $this->environment->getCdOperador();
            $currentTime = $this->environment->getCurrentTime();
            $resultProdutos = [];
            
            foreach($produtos as $produto){
                $qttotlancto =  floatval ( $produto['QTTOTLANCTO']) ;
                $qtlanctoest =  floatval ( $produto['QTTOTLANCTO']  * $produto['VRFATOCONV'] );
                $vrtotlancto=   $produto['VRUNILANCTO'] * floatval ( $produto['QTTOTLANCTO'] );
    
                $produtosaida = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Itlanctoest')->findOneBy(
                    array(  'nrorg' => $produto['NRORG'],
                            'cdfilial' => $produto['CDFILIAL'],
                            'nrsequitem' => $produto['NRSEQUITEM'],
                            'nrlancestq' => $transferencia['NRLANESTSAID'],
                    )
                );
                $produtosaida->setQttotlancto($qttotlancto);
                $produtosaida->setQtlanctoest($qtlanctoest);
                $produtosaida->setVrtotlancto($vrtotlancto);
                $produtosaida->setNrorgultatu($nrorg);
                $produtosaida->setDtultatu($currentTime);
                $produtosaida->setCdoperultatu($operador);
    
                $produtoentrada = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Itlanctoest')->findOneBy(
                    array(  'nrorg' => $produto['NRORG'],
                            'cdfilial' => $produto['CDFILIAL'],
                            'nrsequitem' => $produto['NRSEQUITEM'],
                            'nrlancestq' => $transferencia['NRLANESTENTR'],
    
                    )
                );
                $produtoentrada->setQttotlancto($qttotlancto);
                $produtoentrada->setQtlanctoest($qtlanctoest);
                $produtoentrada->setVrtotlancto($vrtotlancto);
                $produtoentrada->setNrorgultatu($nrorg);
                $produtoentrada->setDtultatu($currentTime);
                $produtoentrada->setCdoperultatu($operador);
            
                $this->entityManager->persist($produtosaida);
                $this->entityManager->flush();
    
                $this->entityManager->persist($produtoentrada);
                $this->entityManager->flush();
                $array = [
                    "qttotlancto" => $qttotlancto,
                    "qtlanctoest" => $qtlanctoest,
                    "vrtotlancto" => $vrtotlancto
                ];
                array_push($resultProdutos, $array);
            }
                $this->atualizaEstoque($transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $transferencia['NRLANESTENTR'], $nrorg);
            $dataSet = new DataSet('produto', $resultProdutos);
            $connection->commit(); 
            return $dataSet;
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function salvarNovaTransferencia($transferencia, $produtos){
        
        try {
            $connection = $this->entityManager->getConnection();
            
            $connection->beginTransaction();
            
            $transferencia['NRORG'] = $this->environment->getNrorgTrab();
            
            $boolContMovEst = ($transferencia["NRCTRLMOVEST"] != '' ||$transferencia["NRLANESTSAID"]||$transferencia["NRLANESTENTR"])?false:true;
            
            $nrctrlmovest = $transferencia["NRCTRLMOVEST"] != '' ? $transferencia["NRCTRLMOVEST"] : $this->geraNovoCodigo('NRTRANSESTQ'.$transferencia['CDFILIAL'],$this->environment->getNrOrgTrab(),1,10);;
            
            //realiza o lançamento de estoque  referente a saída e entrada  ------ 4-Saida 2 -Entrada
            
            $nrlancamentsaida = $transferencia["NRLANESTSAID"] != '' ? $transferencia["NRLANESTSAID"] : $this->saveTabelaLancamentoEstoque($transferencia,$nrctrlmovest,4);;
            
            $nrlancamentoentrada = $transferencia["NRLANESTENTR"] != '' ? $transferencia["NRLANESTENTR"] : $this->saveTabelaLancamentoEstoque($transferencia,$nrctrlmovest,2);;
            
            if($boolContMovEst){
                $dataSet = $this->saveTabelaControleMovimentacaoEstoque($nrctrlmovest,$transferencia,$nrlancamentsaida,$nrlancamentoentrada);
            
                
            }else{
                
                $array = [
                    "nrorg" => $transferencia["NRORG"],
                    "nrlanestsaid" => $nrlancamentsaida,
                    "nrlanestentr" => $nrlancamentoentrada,
                    "nrctrlmovest" => $nrctrlmovest
                ];
                
                $dataSet = new DataSet('transferencia', $array);
            }
            
            foreach($produtos as $produto){
                
                //realiza o lançamento de item de saída e entrada  ------ 4-Saida 2 -Entrada
                $this->saveTabelaItemLancamentoEstoque($nrctrlmovest,$transferencia,[$produto],4,$nrlancamentsaida);
                $this->saveTabelaItemLancamentoEstoque($nrctrlmovest,$transferencia,[$produto],2,$nrlancamentoentrada);
            }
            
            $this->atualizaEstoque($transferencia['CDFILIAL'], $nrlancamentsaida, $nrlancamentoentrada, $transferencia['NRORG']);
            $connection->commit();
            
            return $dataSet;
            
            
            
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function salvarNovaTransferenciaComRequisicao($transferencia, $produtos){
        try {
            $nrrequesto = $transferencia['NRREQUESTO'];
            $transferencia['NRORG'] = $this->environment->getNrOrgTrab();
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $produtos = $this->preparaProdutos($transferencia, $produtos);
            $nrctrlmovest = $this->geraNovoCodigo('NRTRANSESTQ'.$transferencia['CDFILIAL'],$this->environment->getNrOrgTrab(),1,10);
            $transferencia['CDFILIMOVI'] = $transferencia['CDFILIAL'];
            //realiza o lançamento de estoque  referente a saída e entrada  ------ 4-Saida 2 -Entrada
            $nrlancamentsaida    = $this->saveTabelaLancamentoEstoque($transferencia,$nrctrlmovest,4);
            $nrlancamentoentrada = $this->saveTabelaLancamentoEstoque($transferencia,$nrctrlmovest,2);
            $transferencia['NRLANESTSAID'] = $nrlancamentsaida;
            $transferencia['NRLANESTENTR'] = $nrlancamentoentrada;
            
            // valida e atualiza a requisição
            

            $dataSet = $this->saveTabelaControleMovimentacaoEstoque($nrctrlmovest,$transferencia,$nrlancamentsaida,$nrlancamentoentrada);
            //realiza o lançamento de item de saída e entrada  ------ 4-Saida 2 -Entrada
            $this->saveTabelaItemLancamentoEstoque($nrctrlmovest,$transferencia,$produtos,4,$nrlancamentsaida, $nrsequitemrequest = true);
            $this->saveTabelaItemLancamentoEstoque($nrctrlmovest,$transferencia,$produtos,2,$nrlancamentoentrada);
            $this->validateRequisicao($transferencia, $produtos);
            // gera requisição complementar conforme parametrização
            $this->validateReqComp($transferencia, $produtos);
            $this->atualizaEstoque($transferencia['CDFILIAL'], $nrlancamentsaida, $nrlancamentoentrada, $transferencia['NRORG']);
            
            $connection->commit();
            return $dataSet;
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function preparaProdutos($transferencia, $produtos) {
        $nrsequitem = 1;
        $contador = 0;
        foreach ($produtos as $produto) {
            $produtos[$contador]['NRSEQUITEM'] = str_pad($nrsequitem, 4, "0", STR_PAD_LEFT);
            $produtos[$contador]['NRSEQITEMREQC'] = str_pad($nrsequitem, 4, "0", STR_PAD_LEFT);
            $nrsequitem ++;
            $contador++;
        }
        return $produtos;
    }

    public function salvarNovoProduto($transferencia, $produto){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            
            //realiza o lançamento de item de saída e entrada  ------ 4-Saida 2 -Entrada
            $this->saveTabelaItemLancamentoEstoque($transferencia['NRCTRLMOVEST'], $transferencia, [$produto], 4, $transferencia['NRLANESTSAID']);
            $this->saveTabelaItemLancamentoEstoque($transferencia['NRCTRLMOVEST'], $transferencia, [$produto], 2, $transferencia['NRLANESTENTR']);
            
            $this->atualizaEstoque($transferencia['CDFILIAL'], $transferencia['NRLANESTSAID'], $transferencia['NRLANESTENTR'], $transferencia['NRORG']);
            

            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function saveTabelaLancamentoEstoque($transferencia,$nrctrlmovest,$identradaSaida){
        try {
            
            $nrorg = $this->environment->getNrorgTrab();
            $operador = $this->environment->getCdOperador();
            $currentTime = $this->environment->getCurrentTime();
            $nrlancestq = $this->geraNovoCodigo('LANCTOEST'.$transferencia['CDFILIAL'],$nrorg,1,10);
            $lanctoestoq = new Lanctoestoq();
            $lanctoestoq->setNrorg($nrorg);
            $lanctoestoq->setDtinclusao($currentTime);
            $lanctoestoq->setCdoperinclusao($operador);
            $lanctoestoq->setNrorginclusao($nrorg);
            $lanctoestoq->setNrorgultatu($nrorg);
            $lanctoestoq->setDtultatu($currentTime);
            $lanctoestoq->setCdoperultatu($operador);
            $lanctoestoq->setIdativo("S");
            
            $dataLength = strlen($transferencia['DTMOVIESTQ']);
            
            if (strlen($dataLength === 10)){
              $transferencia['DTMOVIESTQ'] = $transferencia['DTMOVIESTQ'].' 00:00:00';  
            }
            $transferencia['DTMOVIESTQ'] = DateUtil::truncateString($transferencia['DTMOVIESTQ']);
            
            $lanctoestoq->setCdfilial($transferencia['CDFILIAL']);
            $lanctoestoq->setNrlancestq($nrlancestq);
            $lanctoestoq->setIdtplancto($identradaSaida);
            $lanctoestoq->setDtlancestq ( DateUtil::getDataDeString($transferencia['DTMOVIESTQ'], 'd/m/Y', true) );
            $lanctoestoq->setDslancestq('Transferência de Estoque '.$nrctrlmovest);
            $lanctoestoq->setCdfilimovi($transferencia['CDFILIMOVI']);
            $lanctoestoq->setCdoperador($operador);

            $this->entityManager->persist($lanctoestoq);
            $this->entityManager->flush();
            
            return $nrlancestq;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveTabelaControleMovimentacaoEstoque($nrctrlmovest,$transferencia,$nrlancamentsaida,$nrlancamentoentrada){
        
        try {
            $nrorg = $this->environment->getNrorgTrab();
            $operador = $this->environment->getCdOperador();
            $currentTime = $this->environment->getCurrentTime();
           
           
            $cdlcestentr = $transferencia['CDLCESTENTR'];
            
            $cdlcestsaid = $transferencia['CDLCESTSAID'];
            if(!$cdlcestentr){
                $cdlcestentr = ' ';
            }
            if(!$cdlcestsaid){
                $cdlcestsaid = ' ';
            }
            
            $ctrlmoviestq = new Ctrlmoviestq();
            $ctrlmoviestq->setNrorg($nrorg);
            $ctrlmoviestq->setDtinclusao($currentTime);
            $ctrlmoviestq->setCdoperinclusao($operador);
            $ctrlmoviestq->setNrorginclusao($nrorg);
            $ctrlmoviestq->setNrorgultatu($nrorg);
            $ctrlmoviestq->setDtultatu($currentTime);
            $ctrlmoviestq->setCdoperultatu($operador);
            $ctrlmoviestq->setIdativo("S");

            $ctrlmoviestq->setNrrequesto( $transferencia['NRREQUESTO'] );
            $ctrlmoviestq->setCdfilial( $transferencia['CDFILIMOVI'] );
            $ctrlmoviestq->setNrctrlmovest($nrctrlmovest);
            $ctrlmoviestq->setCdfilialentr($transferencia['CDFILIAL']);
            $ctrlmoviestq->setNrlanestentr($nrlancamentoentrada);
            $ctrlmoviestq->setCdfilialsaid($transferencia['CDFILIAL']);
            $ctrlmoviestq->setNrlanestsaid($nrlancamentsaida);
            $ctrlmoviestq->setCdalmoxentr($transferencia['CDALMOXENTR']);
            $ctrlmoviestq->setCdalmoxsaid($transferencia['CDALMOXSAID']);
            $ctrlmoviestq->setCdlcestentr($cdlcestentr);
            $ctrlmoviestq->setCdlcestsaid($cdlcestsaid);
            $dataLength = strlen($transferencia['DTMOVIESTQ']);
            if (strlen($dataLength === 10)){
              $transferencia['DTMOVIESTQ'] = $transferencia['DTMOVIESTQ'].' 00:00:00';  
            }
            
            $transferencia['DTMOVIESTQ'] = DateUtil::truncateString($transferencia['DTMOVIESTQ']);
            
            $ctrlmoviestq->setDtmoviestq( DateUtil::getDataDeString($transferencia['DTMOVIESTQ'], 'd/m/Y', true) );

            $this->entityManager->persist($ctrlmoviestq);
            $this->entityManager->flush();
            
            $array = [
                "nrorg" => $nrorg,
                "nrlanestsaid" => $nrlancamentsaida,
                "nrlanestentr" => $nrlancamentoentrada,
                "nrctrlmovest" => $nrctrlmovest
            ];
            $dataSet = new DataSet('transferencia', $array);
            return $dataSet;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveTabelaItemLancamentoEstoque($nrctrlmovest,$transferencia,$produtos,$identradaSaida,$nrlancamentoestoque, $saveNRSEQITEMREQUEST = false){
        try {
            $nrorg       = $this->environment->getNrorgTrab();
            $operador    = $this->environment->getCdOperador();
            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $arrayItens  = [];
            $nrtipocusto = $this->est57000transferenciaestoquefactory->buscaTipoCustoFilial($nrorg,$transferencia['CDFILIAL']);

            $nrtipocusto = $nrtipocusto[0]['CDTIPCUSTEST'];
            
            foreach ($produtos as &$prod) {
                $cdalmoxarife = '';
                $cdlocalestoq = '';
                
                //item de saida
                if($identradaSaida ==4){
                    $cdalmoxarife = $transferencia['CDALMOXSAID'];
                    $cdlocalestoq = $transferencia['CDLCESTSAID'];
                    if(!$cdlocalestoq){
                        $cdlocalestoq =' ';
                    }
                }
                //item de entrada
                else{
                    $cdalmoxarife = $transferencia['CDALMOXENTR'];
                    $cdlocalestoq = $transferencia['CDLCESTENTR'];
                    if(!$cdlocalestoq){
                        $cdlocalestoq =' ';
                    }
                }

                if(!$prod['NRLOTEESTQ']){
                    $prod['NRLOTEESTQ']=' ';
                }

                if(!$prod['NRSUBLOTE']){
                    $prod['NRSUBLOTE']=' ';
                }
                $prod['__is_new'] = false;

                $cdalmoxarifeCusto = $transferencia['CDALMOXSAID'];
                if(!$cdalmoxarifeCusto){
                    $cdalmoxarifeCusto =' ';
                }
                $cdlocalestoqCusto = $transferencia['CDLCESTSAID'];
                if(!$cdlocalestoqCusto){
                    $cdlocalestoqCusto =' ';
                }
                // $vlrunitario  =  $this->est57000transferenciaestoquefactory->gerarValorUnitario($transferencia['CDFILIAL'], $prod['CDPRODUTO'], $nrtipocusto, $transferencia['DTMOVIESTQ'], $this->environment->getCurrentTime()->format("d/m/Y"), $cdalmoxarifeCusto, $cdlocalestoqCusto, $prod['NRLOTEESTQ'], $prod['NRSUBLOTE'])[0]['VRCUSTO'];
                $vlrunitario  =  $prod['VRUNILANCTO'];
                
                $nrsequitem   = $this->est57000transferenciaestoquefactory->geraNrsequitem($nrorg,$transferencia['CDFILIAL'],$nrlancamentoestoque);

                if(isset($prod['NRREQUESTO'])){
                    if($prod['NRREQUESTO'] && $nrsequitem[0]['NRSEQUITEM'] === '0001') {
                        $nrsequitem = $prod['NRSEQUITEM'];
                    }
                    else {
                        $nrsequitem   = $nrsequitem[0]['NRSEQUITEM'];
                    }
                }
                else {
                    $nrsequitem   = $nrsequitem[0]['NRSEQUITEM'];
                }
                $row = new Row();
                $nrsequitemrequest = null;
                if($saveNRSEQITEMREQUEST) {
                    if(isset($prod['NRSEQITEM'])) $nrsequitemrequest = $prod['NRSEQITEM'];
                }

                $row->set('__is_new',true);
                $row->set('NRORG',$nrorg);
                $row->set('NRORGULTATU',$nrorg);
                $row->set('NRORGINCLUSAO',$nrorg);
                $row->set('DTULTATU',$currentTime);
                $row->set('DTINCLUSAO',$currentTime);
                $row->set('CDOPERINCLUSAO',$operador);
                $row->set('CDOPERULTATU',$operador);
                $row->set('IDATIVO','S');
                $row->set('CDOPERADOR',$operador);
                $row->set('CDFILIAL',     $prod['CDFILIAL'] );
                $row->set('NRLANCESTQ',   $nrlancamentoestoque);
                $row->set('NRSEQUITEM',   $nrsequitem);
                $row->set('CDPRODUTO',    $prod['CDPRODUTO']);
                $row->set('CDALMOXARIFE', $cdalmoxarife);
                $row->set('CDLOCALESTOQ', $cdlocalestoq);
                $row->set('DSLANCESTIT',  'Transferência de Estoque '.$transferencia['NRCTRLMOVEST']);
                $row->set('NRLOTEESTQ',   $prod['NRLOTEESTQ']);
                $row->set('NRSUBLOTE',    $prod['NRSUBLOTE']);
                $row->set('CDFILIMOVI',   $prod['CDFILIAL']);
                if(!isset($prod['CDPRODESTO'])){
                    $consulta = $this->est57000transferenciaestoquefactory->getDataForProduct($prod['CDARVPROD']);
                    $prod['CDPRODESTO'] = $consulta['CDPRODESTO'];
                }
                $row->set('CDPRODMOVI',   $prod['CDPRODESTO']);
                $row->set('IDORDEMCLC',   $identradaSaida);
                $row->set('DTLANCMOVI' ,  $prod['DTMOVIESTQ']?? $transferencia['DTMOVIESTQ']);
                $row->set('IDEVOLESTOQ',  'S');
                $row->set('IDTIPOMOVI',   $identradaSaida);
                $row->set('QTTOTLANCTO',  floatval ($prod['QTTOTLANCTO']));
                $row->set('QTLANCTOEST',  floatval ($prod['QTTOTLANCTO']) * ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV']));
                $row->set('VRUNILANCTO' , $vlrunitario);
                $row->set('VRTOTLANCTO' , $vlrunitario * floatval($prod['QTTOTLANCTO']));
                $row->set('VRESTOQDIA' ,  $vlrunitario * floatval($prod['QTTOTLANCTO']));
                $row->set('VRESTOQBRUT' , ($vlrunitario / ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV'])) * floatval($prod['QTTOTLANCTO']));
                $row->set('VRLANCTOEST' , $vlrunitario / ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV']));
                $row->set('VRLANCTOBRUT', $vlrunitario / ($prod['VRFATOCONV'] == 0 ? 1 : $prod['VRFATOCONV']));
                $row->set('NRLANCORIGRP',  null);
                $row->set('NRSEQITEMREQEST',  $nrsequitemrequest);
                $row->set('QTDEVOLESTOQ',  null);
                $arrayItens [] = $row;
            }
            $dataSet = new DataSet('itlanctoest_produto_itrequesto', $arrayItens);
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getAlmoxRequisicao($nrrequesto, $cdfilial) {
        $rows = $this->est57000transferenciaestoquefactory->getAlmoxRequisicao($nrrequesto, $cdfilial);
        $dataSet = new DataSet('getAlmoxRequisicao', $rows);
        return $dataSet;
    }
    
    public function validateReqComp($lancamento, $itens) {
        if($this->shouldCreateReqComp($itens)) {
            $reqComp = $this->getRequisicaoComplementarAberta($lancamento);

            $this->insertItensForReqComp($reqComp, $lancamento, $itens);
        }
    }

    public function insertItensForReqComp($NRREQUESTO, $lancamento, $itens) {
        $params = [
            "NRREQUESTO" => $NRREQUESTO,
            "CDFILIAL" => $lancamento['CDFILIMOVI'],
            "QTDEVOREQU" => 0,
            "IDSTITREQU" => 'A',
            "CDFILILANC" => null,
            "NRLANCESTQ" => null,
            "NRSEQUITEM" => null,
            "CDFILLANCREQC" => $lancamento['CDFILIAL'],
            "NRLANCREQC" => $lancamento['NRLANESTSAID']
        ];
        $nrorg       = $this->environment->getNrorgTrab();
        $operador    = $this->environment->getCdOperador();
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        foreach ($itens as $item) {
            if(Functions::arrayKeyExists('createReqComp',$item) && $item['createReqComp']) {
                $params['NRSEQITREQC'] = $item['NRSEQUITEM'];
                $params['CDPRODUTO'] = $item['CDPRODUTO'];
                $params['NRLOTEESTQ'] = $item['NRLOTEESTQ'];
                $params['NRSUBLOTE'] = $item['NRSUBLOTE'];
                $params['QTREQUESTO'] = $item['QTLANCREQCOMP'];
                $params['NRORG']    = $nrorg;
                $params['NRORGULTATU'   ]    = $nrorg;
                $params['NRORGINCLUSAO']    = $nrorg;
                $params['DTULTATU']    = $currentTime;
                $params['DTINCLUSAO']    = $currentTime;
                $params['CDOPERINCLUSAO']    = $operador;
                $params['CDOPERULTATU']    = $operador;
                $params['IDATIVO'   ]    = 'S';
                $params['CDOPERADOR']    = $operador;



                $newCodeParams = implode('', [$lancamento['CDFILIAL'],
                                  $NRREQUESTO,
                                  $item['CDPRODUTO']]);

                $NRSEQITEM = $this->geraNewSequencial('SEQITREQESTO', $newCodeParams, 4);

                $params['NRSEQITEM'] = $NRSEQITEM;

                $this->est57000transferenciaestoquefactory->insertItrequestoComplementar($params);
            }
        }
    }

    public function shouldCreateReqComp($itens) {
        foreach ($itens as $item) {
            if(Functions::arrayKeyExists('createReqComp',$item) && $item['createReqComp'])
                return true;
        }

        return false;
    }

    public function getRequisicaoComplementarAberta($lancamento) {
        $params = [
            "CDFILLANCREQC" => $lancamento['CDFILIAL'],
            "NRLANCREQC" => $lancamento['NRLANESTSAID']
        ];
        $reqComp = $this->est57000transferenciaestoquefactory->getRequisicaoComplementarAberta($params);

        if(empty($reqComp)) {
            $NRREQUCOMPESTO = $this->geraNewSequencial('NRREQUESTO');

            $params = [
                "CDFILIMOVI" => $lancamento['CDFILIMOVI'],
                "NRREQUESTO" => $lancamento['NRREQUESTO']
            ];
            $reqCompParameters = $this->est57000transferenciaestoquefactory->getRequisicaoComplementarParameters($params);
            $params = [
                "CDFILIAL" => $lancamento['CDFILIAL'],
                "NRREQUESTO" => $NRREQUCOMPESTO,
                "DTREQUESTO" => date("m/d/Y", strtotime($lancamento['DTMOVIESTQ'])),
                "DTPREVREQU" => date("m/d/Y", strtotime($lancamento['DTMOVIESTQ'])),
                "DSREQUESTO" => 'Requisição Complementar - Req. Orig.: ' . $lancamento['NRREQUESTO'] . ' Baixa Orig.:  ' . $lancamento['NRLANESTSAID'],
                "CDSETOR" => $reqCompParameters['CDSETOR'],
                "CDCENTCUST" => $reqCompParameters['CDCENTCUST'],
                "CDOPERREQU" => $this->environment->getCdOperador(),
                "CDALMOXSAID" => $reqCompParameters['CDALMOXSAID'],
                "CDLCESTSAID" => $reqCompParameters['CDLCESTSAID'],
                "CDALMOXENTR" => $reqCompParameters['CDALMOXENTR'],
                "CDLCESTENTR" => $reqCompParameters['CDLCESTENTR'],
                "CDFILLANCREQC" => $lancamento['CDFILIAL'],
                "NRLANCREQC" => $lancamento['NRLANESTSAID']
            ];
            $this->est57000transferenciaestoquefactory->insertRequisicaoComplementar($params);

            return $NRREQUCOMPESTO;
        }

        return $reqComp['NRREQUESTO'];
    }

    /**
     * Essa função é chamda quando está se inserindo uma retirada com uma requisição.
     * Ela tem a finalidade de desabilitar a requisição que foi passada, para que não
     * se possa mais inserir retiradas com aquela requisição que foi passada.
     *
     * @param $lancamento | row com os dados do lançamento
     * @param $itensLancesto | array com rows com os dados de cada item do lançamento
     */
    public function validateRequisicao($lancamento, $itensLancesto) {
        $this->updateItemStatus($lancamento, $itensLancesto);
        
        $newReqStatus = $this->getProperRequisicaoStatus($lancamento);

        $this->updateRequisicaoStatus($lancamento["CDFILIAL"], $lancamento["NRREQUESTO"], $newReqStatus);
    }

    //atualiza os status dos itens de uma requisição na base
    public function updateItemStatus($lancamento, $itensLancesto) {
        $itensRequesto = [];
        foreach($itensLancesto as $itemLancesto) {
            if($itemLancesto['NRREQUESTO'] && $itemLancesto['NRSEQITEM']) {
                $itensRequesto[] = $itemLancesto;
            }
        }

        if($itensRequesto) {
            $this->encerrarItensRequesto($lancamento, $itensRequesto);
        }
    }
    
    public function updateRequisicaoStatus($CDFILIAL, $NRREQUESTO, $IDSTATUSREQ) {
        $IDREQUESTO = ($IDSTATUSREQ == 'A')? 'P' : 'C';
        $params = [
            "IDREQUESTO" => $IDREQUESTO,
            "IDSTATUSREQ" => $IDSTATUSREQ,
            "CDFILIAL" => $CDFILIAL,
            "NRREQUESTO" => $NRREQUESTO
        ];

        $this->est57000transferenciaestoquefactory->updateRequestoStatus($params);
    }

    //lança os CDPRODUTOs pra factory pra atualizar os status dos itens para "E"
    public function encerrarItensRequesto($lancamento, $itensRequesto) {
        $params = [
            "IDSTITREQU" => "E",
            "NRLANCESTQ" => $lancamento["NRLANESTSAID"],
            "NRORG" => $this->nrorg,
            "OLD_NRLANCESTQ" => null
        ];


        foreach ($itensRequesto as $item) {
            $params["CDFILIAL"]   = $item["CDFILIAL"];
            $params["CDFILILANC"] = $item["CDFILIAL"];
            $params["CDPRODUTO"]  = $item["CDPRODUTO"];
            $params["NRSEQITEM"]  = $item["NRSEQITEM"];
            $params["NRSEQUITEM"] = $item["NRSEQUITEM"];
            $params["NRREQUESTO"] = $item["NRREQUESTO"];
            $this->est57000transferenciaestoquefactory->updateItrequestoStatus($params);
        }
    }
    
    public function getProperRequisicaoStatus($lancamento) {
        $params = [
            "CDFILIAL" => $lancamento['CDFILIAL'],
            "NRREQUESTO" => $lancamento['NRREQUESTO'],
            "NRORG" => $this->nrorg,
            "IDSTITREQU" => null
        ];

        $itensRequesto = $this->est57000transferenciaestoquefactory->getItensByRequisicao($params);

        $statusRequesto = $itensRequesto[0]["IDSTITREQU"];
        foreach ($itensRequesto as $itemRequesto) {
            if($itemRequesto["IDSTITREQU"]!= $statusRequesto) {
                $statusRequesto = "P";
            }
        }

        return $statusRequesto;
    }
    
    public function geraNewSequencial($table, $param = '', $colSize = 10) {
        return $this->newCode->getCodeByTableName($table.$param, $this->nrorg, 1, $colSize);
    }
    
    protected function atualizaEstoque($CDFILIAL, $NRLANESTSAID, $NRLANESTENTR, $NRORG) {
        $lancsaida = $this->validacoesEstoqueService->buscaLanctoestoq($CDFILIAL, $NRLANESTSAID, $NRORG);
        $this->validacoesEstoqueService->setLanctoestoq($lancsaida);
        $this->validacoesEstoqueService->atualizaEstoque($CDFILIAL, $NRLANESTSAID);
        $this->validacoesEstoqueService->setLanctoestoq(\Estoque\Entities\Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($CDFILIAL, $NRLANESTENTR, $NRORG));
        $this->validacoesEstoqueService->atualizaEstoque($CDFILIAL, $NRLANESTENTR);
    }
    
    public function buildObjectTheCode($code, $params) {
        
        $pointer = 0;
        $objectRead = [];
        $boolUsaSeparador = $params['IDTPSEPARADOR'] <> 2;
        
        //$data = substr($code, $pointer, strlen(str_replace('/', '', $params["DSFORMDATA"])));
        
        $data = substr($code, $pointer, strlen($params["DSFORMDATA"]));
        //$data = str_replace('/', '', $data);
        
        if(is_null($data) || $data == false){
            $data = \DateTime::createFromFormat('dmY', substr($code, $pointer, strlen($params["DSFORMDATA"])));
        }
        
        if(!$data){
            throw new \Exception("Formato de Data Inválida ou Parametrização Invalida");
        }
        
        //$data = $data->format(Dateutil::FORMATO_BRASILEIRO);
        
        
        $objectRead["DTMOVIESTQ"] = $data; //Lê a data
        $pointer += strlen($params["DSFORMDATA"]);
        
        //Lê o campo que identifica o produto
        $objectRead[$params["PRODUTO"]] = substr($code, $pointer, $params["QTDIGPRODUTO"]);
        $pointer += $params["QTDIGPRODUTO"];

        //Lê a quantidade (inteira)
        $int = substr($code, $pointer, $params["QTNUMEROINT"]);
        $pointer += $params["QTNUMEROINT"];
        //Confere se usa separador decimal
        $separador = substr($code, $pointer, 1);
        if($boolUsaSeparador){
            if(($separador != "," && $params["IDTPSEPARADOR"] == 0) || ($separador != "." && $params["IDTPSEPARADOR"] == 1)){
                throw new \Exception("Separador informado ( $separador ) não corresponde com a parametrização definida.");
            }
            $pointer += 1;
        }else{
            if(($separador == "," || $separador == ".") && ($params["IDTPSEPARADOR"] == 2)){
                throw new \Exception("Separador informado ( $separador ) não corresponde com a parametrização definida.");
            }
            $pointer += 0;
        }
        //Lê a quantidade (decimal)
        $decimal = substr($code, $pointer, $params["QTNUMERODEC"]);
        $pointer += $params["QTNUMERODEC"];
        
        //Seta Quantidade
        $objectRead["QTTOTLANCTO"] = $int || $decimal ? floatval($int . '.' . $decimal) : 0;

        //Lê o almoxarficado
        $objectRead["CDALMOXARIFE"] = substr($code, $pointer, $params["QTTAMALMOX"]);
        $pointer += $params["QTTAMALMOX"];
        //Lê o localização
        $objectRead["CDLOCALESTOQ"] = substr($code, $pointer, $params["QTTAMLOCAL"]);
        $pointer += $params["QTTAMLOCAL"];

        //Lê o lote
        $objectRead["NRLOTEESTQ"] = substr($code, $pointer, $params["QTTAMLOTE"]);
        $pointer += $params["QTTAMLOTE"];

        //Lê o sublote
        $objectRead["NRSUBLOTE"] = substr($code, $pointer, $params["QTTAMSUBLOTE"]);
        $pointer += $params["QTTAMSUBLOTE"];
        
        return $objectRead;
    }
    
    public function completaProduto($produto, $cdfilial, $cdfilimovi, $dtLancEst, $ErrorRowNumber = null, $fileType){
        $produtoTabela = $this->getProdutoTabela($produto, $cdfilimovi);
        $codImportado = isset($produto['CDARVPROD'])? ($produto['CDARVPROD']) : (isset($produto['CDPRODUTO'])? $produto['CDPRODUTO'] : $produto['CDBARPRODUTO']);
        if(!$produtoTabela){
            if(empty($codImportado))
                throw new \Exception("Filial sem parametrização Para importação");
            else
                throw new \Exception("Produto não encontrado. Codigo: ".$codImportado);
        }
         // validações de controle de lote
        $nrorg = $this->environment->getNrOrgTrab();
        $paramfilial = Paramfilial::findOneByCdfilialAndNrorg($cdfilimovi, $nrorg);
        
        $tpCusto = $paramfilial->getCdtipcustest();
        $utAlmox = $paramfilial->getIdutilalmox();
        $utLcEst = $paramfilial->getIdutillcestq();
        $utlote = $paramfilial->getIdutillote();
        
        $prodfili = Prodfili::findOneByCdprodutoAndCdfilialAndNrorg($produtoTabela["CDPRODMOVI"], $cdfilimovi, $nrorg);
        $prodUtLote  = $prodfili->getIdutctrllote();
        $prodAlmox   = $prodfili->getCdalmoxarife();
        $prodLocal   = $prodfili->getCdlocalestoq();
        
        if($utAlmox == 'S' && !isset($produto["CDALMOXARIFE"]) && !isset($produtoTabela["CDALMOXARIFE"]))
            throw new \Exception('Filial controla almoxarifado e o mesmo não foi encontrado no arquivo.');
        if($utLcEst == 'S' && !isset($produto["CDLOCALESTOQ"]) && !isset($produtoTabela["CDLOCALESTOQ"]))
            throw new \Exception('Filial controla local de estoque e o mesmo não foi encontrado no arquivo.');
        if($utlote == 'S' && $prodUtLote == 'S' && !isset($produto["NRLOTEESTQ"])  && !isset($produtoTabela["NRLOTEESTQ"])){
            throw new \Exception('Filial controla lote e o mesmo não foi encontrado no arquivo.');
        }
        
        $qttotlancto = isset($produto["QTTOTLANCTO"]) ? (int) $produto["QTTOTLANCTO"] : 0;

        if($qttotlancto <= 0){
            throw new \Exception('Preencha o campo QTTOTLANCTO (Quantidade Unitária) na linha ' .  $ErrorRowNumber);
        }
        $vrunit = isset($produto["VRUNILANCTO"]) ? $produto["VRUNILANCTO"] : $produtoTabela["VRUNILANCTO"];
        $vrunitbrut = isset($produto["VRUNITBRUT"]) ? $produto["VRUNITBRUT"] : $produtoTabela["VRUNITBRUT"];
        $vrtotlancto = $vrunit * $qttotlancto;
        $vrlanctobrut = $vrunitbrut * $qttotlancto;
        $vrfatoconv = $produtoTabela['VRFATOCONV']? $produtoTabela['VRFATOCONV'] : 1;
        $qtlanctoest = $qttotlancto * $vrfatoconv;
        $produtoTabela["CDFILIAL"] = $cdfilial;
        $produtoTabela["CDFILIMOVI"] = $cdfilimovi;
        $produtoTabela["DTMOVIESTQ"] =  isset($produto["DTMOVIESTQ"]) ? $produto["DTMOVIESTQ"] : $dtLancEst;
        $produtoTabela["CDPRODUTO"] = isset($produto["CDPRODUTO"]) ? $produto["CDPRODUTO"] : $produtoTabela["CDPRODUTO"];
        $produtoTabela["CDARVPROD"] = isset($produto["CDARVPROD"]) ? $produto["CDARVPROD"] : $produtoTabela["CDARVPROD"];
        $produtoTabela["CDALMOXARIFE"] = (isset($produto["CDALMOXARIFE"]) && !empty($produto["CDALMOXARIFE"])) ? $produto["CDALMOXARIFE"] : ($prodAlmox? $prodAlmox : ' ');
        $produtoTabela["CDLOCALESTOQ"] = (isset($produto["CDLOCALESTOQ"]) && !empty($produto["CDLOCALESTOQ"])) ? $produto["CDLOCALESTOQ"] : ($prodLocal? $prodLocal : ' ');
        $produtoTabela["NRLOTEESTQ"] = isset($produto["NRLOTEESTQ"]) ? $produto["NRLOTEESTQ"] : $produtoTabela["NRLOTEESTQ"];
        $produtoTabela["NRSUBLOTE"] = isset($produto["NRSUBLOTE"]) ? $produto["NRSUBLOTE"] : $produtoTabela["NRSUBLOTE"];
        $produtoTabela["QTTOTLANCTO"] = $qttotlancto;
        $produtoTabela["VRUNILANCTO"] = $vrunit;
        $produtoTabela["VRUNITBRUT"] = $vrunitbrut;
        $produtoTabela["QTLANCTOEST"] = $qtlanctoest;
        
        $produtoTabela["NRLOTEESTQ"] = str_replace(' ', '', $produtoTabela["NRLOTEESTQ"]);
        $produtoTabela["CDLOCALESTOQ"] = str_replace(' ', '', $produtoTabela["CDLOCALESTOQ"]);
        
        $custo = $this->validacoesEstoqueService->consultaCustoQuantidadeProduto($cdfilimovi, $produtoTabela['CDARVPROD'], $produtoTabela['DTMOVIESTQ'], $tpCusto, $produtoTabela['CDALMOXARIFE'], $produtoTabela['CDLOCALESTOQ'], $produtoTabela['NRLOTEESTQ'], $produtoTabela['NRSUBLOTE']); 
        
        if ($fileType === 'xls'){
            $produtoTabela["CDALMOXSAID"] = $produto["CDALMOXSAID"];
            $produtoTabela["CDALMOXENTR"] = $produto["CDALMOXENTR"];
            $produtoTabela["CDLCESTSAID"] = $produto["CDLCESTSAID"];
            $produtoTabela["CDLCESTENTR"] = $produto["CDLCESTENTR"];
        }
        
        return $produtoTabela;
    }
    
    public function getProdutoTabela($produto, $cdfilial){

        if(isset($produto["CDARVPROD"])){
            $parameters = array(
                "P_CDARVPROD" => str_replace(" ", "", $produto["CDARVPROD"])
            );
        } else if(isset($produto["CDBARPRODUTO"])){
            $parameters = array(
                "P_CDBARPRODUTO" => str_replace(" ", "", $produto["CDBARPRODUTO"])
            );
        } else{
            $parameters = array(
                "P_CDPRODUTO" => str_replace(" ", "", $produto["CDPRODUTO"])
            ); 
        }
        $parameters["P_CDFILIAL"] = $cdfilial;
        return $this->est57000transferenciaestoquefactory->getProdutoTabela($parameters);
    }
    
    public function readXls($row){
        $file = $row['FILE'];
        
        $columnNames = $row['CAMPOS'];
        
        
        $sheetsToLoad = array('Sheasdasdsadet1', 'Sheet1', 'Planilha1');
        
        $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name']);
        
        if(!$xlsFile){
            
            throw new \Exception(self::IMPORT_LOAD_FILE_ERROR);
        }
        
        $worksheet = $xlsFile->getActiveSheet(); 
        
        
        
        $table = FileReader::readWorksheetTable($worksheet, $columnNames);
        
        
        return $table;
    }
    
    public function formatXls(Table $table){
        $columnsReference = $table->getColumnsReference();
        $tableRows = $table->getRows();
        $columnsNumber = $table->getNumberOfColumns();
        $i = 0;
    
        foreach ($tableRows as &$row) {
            $row = $row['content'];
            for($column = 0; $column < $columnsNumber; $column++){
                $columnLetterIndex = Coordinate::stringFromColumnIndex($column+1);

                $columnName = $columnsReference[$columnLetterIndex];
                
                $row[$columnName] = self::formatValue($row[$columnName], $columnName);
            }
            $row["ROWNUM"] = $i;

            $i++;
        }
        return $tableRows;
    }
    
    private function formatValue($value, $columnName){
        switch($columnName){
            case $columnName == 'CDBARPRODUTO':
            case $columnName == 'CDARVPROD':
            case $columnName == 'CDPRODUTO':   
                $value = self::cleanSpecialCharacters(strval($value));
                break;
            case $columnName == 'NRLOTEESTQ':
            case $columnName == 'NRSUBLOTE':
          
                break;
            default: 
                $value = strval($value);
        }
        return $value;
    }
    
    public function cleanSpecialCharacters($string){
        $string = str_replace('.', '', $string); 
        return preg_replace('/[^A-Za-z0-9\-]/', '', $string);
    }
    
    public function buildObjectTheCodeXls($produto, $params, &$itensWithErrors, $ErrorRowNumber) {
      return [
            $params["PRODUTO"] => $produto[$params["PRODUTO"]],
            "NRLOTEESTQ" => $produto['NRLOTEESTQ'] ?? ' ',
            "DTMOVIESTQ" => $produto['DTMOVIESTQ'] ?? ' ',
            "NRSUBLOTE" => $produto['NRSUBLOTE'] ?? ' ',
            "QTTOTLANCTO" => $produto['QTTOTLANCTO'], 
            "CDALMOXARIFE" => $produto['CDALMOXSAID'] ?? ' ',
            "CDLOCALESTOQ" => $produto['CDLCESTSAID'] ?? ' ',
            "CDALMOXSAID" => $produto['CDALMOXSAID'] ?? ' ',
            "CDALMOXENTR" => $produto['CDALMOXENTR'] ?? ' ',
            "CDLCESTSAID" => $produto['CDLCESTSAID'] ?? ' ',
            "CDLCESTENTR" => $produto['CDLCESTENTR'] ?? ' ',
            "NRLOTEESTQ" => $produto['NRLOTEESTQ'] ?? ' ',
            "NRSUBLOTE" => $produto['NRSUBLOTE']  ?? ' '
            
        ];
        
    }
    
} 